@@ -27,9 +27,9 @@
  */
 static inline bool cyaml__host_is_little_endian(void)
 {
-	static const uint16_t test = 1;
+	const uint16_t test = 1;
 
-	return ((const uint8_t *) &test)[0] == 1;
+	return ((const uint8_t *) &test)[0];
 }
 
 /**
